#################################################################################################################
# Говорливый модуль Megahal под редакцией Buster.
# Корректная работа допустима при использовании патча Suzi Project.
#################################################################################################################
# Установка:
# 1) Установка модуля
#   а) Для OS семейства Unix необходимо каталог megahal.mod поместить в папку src/mod папки исходников бота
#      и подать набор стандартных команд для компиляции. И в зависимости от метода компиляции при необходимости
#      файл megahal.so положить в папку modules папки бота.
#   b) Для OS семейства Windows необходимо файл megahal.dll положить в папку modules папки бота.
# 2) Папку megahal (с словарями и скриптами) положить в папку бота либо в папку со скриптами.
# 3) В скрипте megahal.tcl указать правильно путь к папке, которая была скопирована в пункте 2. Так же, по
#    желанию, можно отредактировать остальные настройки.
# 3) Прописать в конфигурационном файле путь к файлу megahal.tcl. Пример source megahal/megahal.tcl или
#    source scripts/megahal/megahal.tcl в зависимости от месторасположения.
# 4) Сделать перезагрузку конфигурационного файла, либо перезагрузку бота командами .rehash .restart
# 5) Для включения разговора/обучения на канале необходимо настроить следующие параметры:
#       meg-dialog      - ведение диалога, при обращении по нику бота
#       meg-freespeak   - свободный режим, будет отвечать на случайное сообщение
#       meg-learndialog - обучаться в диалоге
#       meg-learnall    - обучаться от любых сообщений
#    Для включения выбранной настройки следует ввести ..chanset #канал +параметр
#    Для отключения .chanset #канал -параметр
#    Например: .chanset #канал +meg-dialog
#    Так же есть два параметра, которые задают промежуток времени для вывода ответа
#      meg-min-freespeak - минимальное количество сообщений в пределах которого бот не будет выдавать сообщений
#                          во freespeak режиме
#      meg-max-freespeak - максимальное количество сообщений в пределах которого бот выдать реплику на одно из
#                          сообщений в режиме freespeak.
#    Значение по умолчанию 15 и 50 соответственно. Изменить можно командой .chanset #канал параметр время
#    Например: .chanset #канал meg-min-freespeak 20
#################################################################################################################
# Оригинальный алгоритм и база антимата взята от sbw.tcl by Dark_Agent
#################################################################################################################

namespace eval megahal {
    
    #############################################################################################################
    # Версия и автор скрипта
    variable author     "Buster <buster@ircworld.ru>"
    variable version    "4.0.2"
    variable date       "11-DEC-2007"
    
    variable megahal
    global meg_patch
    global meg_file_prefix
    
    set megahal(list_botnick)   [list]
    
    #############################################################################################################
    # Префикс между ником и текстом при ответе.
    # Пример: ": ", тоесть при ответе если бот обращается к нику будет поставлен этот префикс. 
    # [09:29] <AliasNET> Buster: Привет тебе.
    set megahal(pref_out)       ": "
    
    #############################################################################################################
    # Минимальное и максимальное время задержки в ответе, в милисекундах. Тоесть бот будет отвечать с задержкой
    # из данного промежутка времени.
    set megahal(min_time_out)   1000
    set megahal(max_time_out)   5000
    
    #############################################################################################################
    # Путь к папке с файлами словарей и параметров. Пример: "./megahal", если папка megahal лежит в корне бота;
    # или "scripts/megahal", если папка лежит в папке со скриптами.
    set ::meg_patch             "./megahal"
    
    #############################################################################################################
    # Префикс файлов словарей. Имя файлов словарей без расширения, обычно "megahal".
    set ::meg_file_prefix       "megahal"
    
    #############################################################################################################
    # Время, в минутах, через которое следует сохранять и делать бэкап словарей. При этом формируюся файы:
    # xxx.brn     - основной словарь;
    # xxx.brn~bak - бэкап базы от прошлого сохранения;
    # xxx.msg     - список фраз которые содержатся в словаре (нужен только для визуального просмотра фраз);
    # xxx.        - словарь со списком ключевых слов;
    set megahal(time_save)      60
    
    #############################################################################################################
    # Флаги юзеров, которых бот должен игнорить. Если пользователь добавлен в юзерлист то прописать флаг
    # исключения можно командой .chattr ник +I, либо если это бот добавив его как бота .+bot ник хост
    set megahal(ignoreflag)     "bI"
    
    #############################################################################################################
    # Защита от флуда, задается двумя параметрами xx:yy, где хх - количество обращений за yy - секунд, если
    # количество допустимых сообщений превышает то бот будет блокировать пользователя.
    # При значении 0:0 - флуд контроль отключен.
    set megahal(flood)          3:4
    
    #############################################################################################################
    # Время, в секундах, на сколько бот будет блокировать пользователя за флуд.
    set megahal(time_ignore)    180
    
    #############################################################################################################
    # Игнорировать сообщения на канале если оно начинается с указанных символов
    set megahal(chan_ignore_text)   [list "!" "$" "%" "&" "."]
    
    #############################################################################################################
    # Игнорировать сообщения в привате если оно начинается с указанных символов
    set megahal(msg_ignore_text)    [list "!" "$" "%" "&" "." "auth" "PASS" "addmask"]
    
    #############################################################################################################
    # Сохранять логи диалогов chan - канальных, msg - приватных (1 - сохранять, 0 - не сохранять)
    set megahal(save_log_chan)  1
    set megahal(save_log_msg)   1
    
    #############################################################################################################
    # Путь к файлам логов. Значение по умолчанию "$meg_patch/megahallog" тоесть логи будут сохранятся в папку 
    # megahallog находящуюся в папке megahal. Позабодтесь о том чтобы данная папка существовала.
    set megahal(log_patch)      "$meg_patch/megahallog"
    
    #############################################################################################################
    # Опция управления антиматом:
    #  0 - антимат отключен
    #  1 - будет игнорироватся запоминание фраз, но отвечать бот на такие фразы будет
    #  2 - будут полностью игнорироваться сообщения содержащие мат.
    set megahal(antimat)        1
    
    #############################################################################################################
    # Включение/отключение модуля на диалог в приватных сообщениях (1 - включить, 0 - отключить)
    set megahal(meg_msg)        1
    
    #############################################################################################################
    # Обучаться в приватных разговорах (1 - обучаться, 0 - отключить)
    set megahal(meg_learn)      0
    
    #############################################################################################################
    # Уровень отладки. Задается числом от 0 до 4. Соответсвенно 0 - отключено, 4 - максимальная информация
    set megahal(debug_level)    3
    
    #############################################################################################################
    #############################################################################################################
    #############################################################################################################
    
    #############################################################################################################
    # Объявление флагов
    
    setudef flag meg-dialog
    setudef flag meg-freespeak
    setudef flag meg-learndialog
    setudef flag meg-learnall
    setudef int meg-min-freespeak
    setudef int meg-max-freespeak
    
    #############################################################################################################
    # Процедуры обработки канальных и приватных сообщений
    
    proc pubm {nick uhost hand chan text} {
        variable megahal
        
        set meg_dialog [channel get $chan meg-dialog]
        set meg_freespeak [channel get $chan meg-freespeak]
        set meg_learndialog [channel get $chan meg-learndialog]
        set meg_learnall [channel get $chan meg-learnall]
        
        if {!$meg_dialog && !$meg_freespeak && !$meg_learnall} return
        if {[matchattr $hand $megahal(ignoreflag)]} return
        if {[lfindfirst $megahal(chan_ignore_text) $text]} return
        
        set text [stripcodes bcru $text]
        
        #if {![regexp -- {^[^\w]*(([\w]+)[^\w]*(.*?))$} $text -> t1 t2 t3]} return
        #if {![regexp -nocase -- {[a-zа-я]} $t3]} return
        if {![regexp -- {^[^\w]*((.*?)[^\w]*(?:\ +)[^\w]*(.*?))$} $text -> t1 t2 t3]} return
        
        if {$meg_freespeak} {
            if {![info exists megahal(freespeak_ind,$chan)]} {
                set megahal(freespeak_ind,$chan) [get_freespeak_skip $chan]
            }
            incr megahal(freespeak_ind,$chan) -1
            out_debug "Канал: $chan. Количество сообщений до свободного вывода: $megahal(freespeak_ind,$chan)" 4
        }
        
        if {$megahal(antimat) > 0} {
            set matdetected [DetectMat $text]
            if {$matdetected && $megahal(antimat) > 0} {out_debug "Обнаружен мат в сообщение" 2}
            if {$matdetected && $megahal(antimat) > 1} {return}
        } else {
            set matdetected 0
        }
        
        if {$meg_dialog && [lequal [get_lbotnick] $t2]} {
            
            if {[userisignore $nick]} {
                out_debug "Ник $nick заблокирован, сообщение проигнорировано" 2
                return
            }
            if {[detectflood $chan $nick]} {
                out_debug "Замчен флуд от $nick, сообщение проигнорировано" 2
                addignore $nick
                return
            }
            
            out_debug "Канал: $chan. Обращение к нику бота." 3
            set reply [getreply $t3]
            out_debug "Получение реплики: ($t3) ($reply)" 3
            out_msg $chan "$nick$megahal(pref_out)$reply"
            save_logfile chan user $chan $nick $text
            save_logfile chan bot $chan $nick "$nick$megahal(pref_out)$reply"
            
            if {$meg_learndialog && ![lmatch [chanlist $chan] $t3] && !($matdetected && $megahal(antimat) > 0)} {
                out_debug "Запоминание фразы t3 ($t3)" 3
                learn $t3
            }
            
        } elseif {$meg_dialog && [lmatch [get_lbotnick] $t1]} {
            
            if {[userisignore $nick]} {
                out_debug "Ник $nick заблокирован, сообщение проигнорировано" 2
                return
            }
            if {[detectflood $chan $nick]} {
                out_debug "Замчен флуд от $nick, сообщение проигнорировано" 2
                addignore $nick
                return
            }
            
            out_debug "Канал: $chan. Ник бота найден в тексте." 3
            set reply [getreply $t1]
            out_debug "Получение реплики: ($t1) ($reply)" 3
            out_msg $chan $reply
            save_logfile chan user $chan $nick $text
            save_logfile chan bot $chan $nick $reply
            
        } elseif {$meg_freespeak && $megahal(freespeak_ind,$chan) < 1} {
            
            out_debug "Канал: $chan. Свободный режим вывода." 3
            set reply [getreply $t1]
            out_debug "Получение реплики: ($t1) ($reply)" 3
            out_msg $chan $reply
            save_logfile chan user $chan $nick $text
            save_logfile chan bot $chan $nick $reply
            
            if {!($matdetected && $megahal(antimat) > 0)} {
                if {$meg_learnall && [lequal [chanlist $chan] $t2] && ![lmatch [chanlist $chan] $t3]} {
                    out_debug "Запоминание фразы t3 ($t3)" 3
                    learn $t3
                } elseif {$meg_learnall && ![lmatch [chanlist $chan] $t1]} {
                    out_debug "Запоминание фразы t1 ($t1)" 3
                    learn $t1
                }
            }
            
            set megahal(freespeak_ind,$chan) [get_freespeak_skip $chan]
            
        } elseif {$meg_learnall} {
            
            if {!($matdetected && $megahal(antimat) > 0)} {
                if {[lequal [chanlist $chan] $t2] && ![lmatch [chanlist $chan] $t3]} {
                    out_debug "Запоминание фразы t3 ($t3)" 3
                    learn $t3
                } elseif {![lmatch [chanlist $chan] $t1]} {
                    out_debug "Запоминание фразы t1 ($t1)" 3
                    learn $t1
                }
            }
            
        }
        
    }
    
    proc msgm {nick uhost hand text} {
        variable megahal
        
        if {!$megahal(meg_msg)} return
        if {[matchattr $hand $megahal(ignoreflag)]} return
        if {[lfindfirst $megahal(msg_ignore_text) $text]} return
        set text [stripcodes bcru $text]
        
        if {![regexp -- {^[^\w]*((.*?)[^\w]*(?:\ +)[^\w]*(.*?))$} $text -> t1 t2 t3]} {
            if {![regexp -- {^[^\w]*(.*?)$} $text -> t1]} return
            set t2 ""
            set t3 ""
        }
        
        if {$megahal(antimat) > 0} {
            set matdetected [DetectMat $text]
            if {$matdetected && $megahal(antimat) > 0} {out_debug "Обнаружен мат в сообщение" 2}
            if {$matdetected && $megahal(antimat) > 1} {return}
        } else {
            set matdetected 0
        }
        
        if {[lequal [get_lbotnick] $t2]} {
            
            if {[userisignore $nick]} {
                out_debug "Ник $nick заблокирован, сообщение проигнорировано" 2
                return
            }
            if {[detectflood "msg" $nick]} {
                out_debug "Замчен флуд от $nick, сообщение проигнорировано" 2
                addignore $nick
                return
            }
            if {![regexp -nocase -- {[a-zа-я]} $t3]} return
            
            out_debug "Приват: $nick. Обращение к нику бота." 3
            set reply [getreply $t3]
            out_debug "Получение реплики: ($t3) ($reply)" 3
            out_msg $nick $reply
            save_logfile msg user "" $nick $text
            save_logfile msg bot "" $nick $reply
            
            if {$megahal(meg_learn) && ![lmatch [get_lbotnick] $t3] && !($matdetected && $megahal(antimat) > 0)} {
                out_debug "Запоминание фразы t3 ($t3)" 3
                learn $t3
            }
            
        } else {
            
            if {[userisignore $nick]} {
                out_debug "Ник $nick заблокирован, сообщение проигнорировано" 2
                return
            }
            if {[detectflood "msg" $nick]} {
                out_debug "Замчен флуд от $nick, сообщение проигнорировано" 2
                addignore $nick
                return
            }
            
            out_debug "Приват: $nick." 3
            set reply [getreply $t1]
            out_debug "Получение реплики: ($t1) ($reply)" 3
            out_msg $nick $reply
            save_logfile msg user "" $nick $text
            save_logfile msg bot "" $nick $reply
            
            if {$megahal(meg_learn) && ![lmatch [get_lbotnick] $t1] && !($matdetected && $megahal(antimat) > 0)} {
                out_debug "Запоминание фразы t1 ($t1)" 3
                learn $t1
            }
            
        }
        
    }
    
    #############################################################################################################
    # Процедуры получения параметров
    
    proc get_freespeak_skip {chan} {
        
        set min_freespeak [channel get $chan meg-min-freespeak]
        if {$min_freespeak < 1} {
            set min_freespeak 15
            channel set $chan meg-min-freespeak $min_freespeak
        }
        set max_freespeak [channel get $chan meg-max-freespeak]
        if {$max_freespeak < 1} {
            set max_freespeak 50
            channel set $chan meg-max-freespeak $max_freespeak
        }
        
        return [expr int(rand() * ($max_freespeak - $min_freespeak)) + $min_freespeak]
        
    }
    
    proc get_lbotnick {} {
        variable megahal
        global botnick 
        
        return [concat $botnick $megahal(list_botnick)]
        
    }
    
    #############################################################################################################
    # Процедуры вывода сообщений
    
    proc out_msg {chan text} {
        variable megahal
        
        after [expr int(rand() * ($megahal(max_time_out) - $megahal(min_time_out))) + $megahal(min_time_out)] \
            [list putserv "PRIVMSG $chan :$text"]
        
    }
    
    proc out_error {text} {
        putlog "[namespace current]:: Error: $text"
    }
    
    proc out_debug {text {level 1}} {
        variable megahal
        if {$megahal(debug_level) >= $level} {
            putlog "[namespace current]:: Debug: $text"
        }
    }
    
    #############################################################################################################
    # Процедуры загрузки баз
    
    proc load_base {} {
        global meg_file_prefix meg_patch
        variable megahal
        
        set megahal(list_botnick) [list]
        
        set filenick "$meg_patch/$meg_file_prefix.nick"
        if {[file exists $filenick]} {
            if {[catch {
                set fileio [open $filenick r] 
                set data [read $fileio]
                close $fileio
                set lines [split $data \n]
                set count 0
                foreach line $lines {
                    set line [string trim $line]
                    if {[string index $line 0] != "#" && ![string is space $line]} {
                        incr count
                        lappend megahal(list_botnick) $line
                    }
                }
                out_debug "Файл '$filenick' загружен. Ников найдено: $count" 1
            } errMsg]} {
                out_error "Загрузка файла '$filenick' ($errMsg)."
            }
        } else {
            out_error "Файл '$filenick' не найден."
        }
        
        set megahal(BadWords) [list]
        
        set filebw "$meg_patch/$meg_file_prefix.bw"
        if {[file exists $filebw]} {
            if {[catch {
                set fileio [open $filebw r] 
                set megahal(BadWords) [read $fileio]
                close $fileio
                #set megahal(BadWords) [split $data \n]
                set count [llength $megahal(BadWords)]
                out_debug "Файл '$filebw' загружен. Масок найдено: $count" 1
            } errMsg]} {
                out_error "Загрузка файла '$filebw' ($errMsg)."
            }
        } else {
            out_error "Файл '$filebw' не найден."
        }
        
        set megahal(BadExepts) [list]
        
        set filebwe "$meg_patch/$meg_file_prefix.bwe"
        if {[file exists $filebwe]} {
            if {[catch {
                set fileio [open $filebwe r] 
                set megahal(BadExepts) [read $fileio]
                close $fileio
                #set megahal(BadExepts) [split $data \n]
                set count [llength $megahal(BadExepts)]
                out_debug "Файл '$filebwe' загружен. Масок найдено: $count" 1
            } errMsg]} {
                out_error "Загрузка файла '$filebwe' ($errMsg)."
            }
        } else {
            out_error "Файл '$filebwe' не найден."
        }
        
    }
    
    #############################################################################################################
    # Процедуры логирования
    
    proc save_logfile {pref type chan nick text} {
        global botnick
        variable megahal
        
        set filename $pref
        if {$pref == "chan"} {
            if {!$megahal(save_log_chan)} return
            append filename ".[get_filename $chan]"
        } elseif {$pref == "msg"} {
            if {!$megahal(save_log_msg)} return
            append filename ".[get_filename $nick]"
        }
        append filename ".log"
        
        if {$type == "bot"} {
            set out_text "\[[clock format [unixtime] -format %d/%m/%y-%H:%M:%S]\] <$botnick> $text"
        } elseif {$type == "user"} {
            set out_text "\[[clock format [unixtime] -format %d/%m/%y-%H:%M:%S]\] <$nick> $text"
        }
        
        if {[catch {
            set fileio [open $megahal(log_patch)/$filename a+]
            puts $fileio $out_text
            flush $fileio
            close $fileio
        } errMsg]} {
            out_error "Неудалось записать файл '$megahal(log_patch)/$filename' ($errMsg)."
        }
        
    }
    
    #############################################################################################################
    # Дополнительные процедуры и функции
    
    proc get_filename {text} {
        return [regsub -all -- {[\\\/\:\*\?\"\<\>\|]} $text {_}]
    }
    
    proc lequal {listn text} {
        
        foreach line $listn {
            if {[string equal -nocase $line $text]} {
                return 1
            }
        }
        return 0
        
    }
    
    proc lmatch {listn text} {
        
        foreach line $listn {
            regsub -all {[][${}^?+*()|.\\]} $line {\\\0} line
            if {[string match -nocase *$line* $text]} {
                return 1
            }
        }
        return 0
        
    }
    
    proc lfindfirst {listn text} {
        
        foreach line $listn {
            if {[string equal -nocase [string range $text 0 [expr [string length $line]-1]] $line]} {return 1}
        }
        return 0
        
    }
    
    proc text_has_rus {text} {
        set len [string length $text]
        set cnt 0
        while {$cnt < $len} {
            if {[is_rus [string index $text $cnt]]} {return 1}
            incr cnt
        }
        return 0
    }
    
    proc is_rus {letter} {
        set caps [list Й Ц У К Е Н Г Ш Щ З Х Ъ Ф Ы В А П Р О Л Д Ж Э Я Ч С М И Т Ь Б Ю Ё й ц у к е н г ш щ з х ъ ф ы в а п р о л д ж э я ч с м и т ь б ю ё]
        if {[lsearch -exact $caps $letter] > -1} {return 1} else {return 0}
    }
    
    proc lat2rus {text} {
        return [string map -nocase {q й w ц e у r к t е y н u г i ш o щ p з [ х ] ъ a ф s ы d в f а g п h р j о k л l д ; ж ' э z я x ч c с v м b и n т m ь , б ... ю / . Q Й W Ц E У R К T Е Y Н U Г I Ш O Щ P З \{ Х \} Ъ A Ф S Ы D В F А G П H Р J О K Л L Д : Ж \" Э Z Я X Ч C С V М B И N Т M Ь < Б > Ю ? , ^ : & ? ` ё ~ Ё $ ;} $text] 
    }
    
    proc remove_duplicates {text} {
        set res ""
        set ch ""
        for {set i 0} {$i < [string length $text]} {incr i} {
            set newch [string index $text $i]
            if {$ch != $newch} {
                append res $newch
            }
            set ch $newch
        }
        return $res
    }
    
    proc remove_symbols {text} {
        return [regsub -all {([№\`\~\!\@\#\$\%\^\&\*\(\)\_\-\+\=\{\}\[\]\;\:\'\"\<\>\.\,\/\?\|\\])+} $text " "]
    }
    
    #############################################################################################################
    # Процедуры контроля за флудом
    
    proc detectflood {chan nick} {
        variable megahal
        
        set lflood [split $megahal(flood) :]
        set fmcount [lindex $lflood 0]
        set ftime [lindex $lflood 1]
        
        if {[string is space $fmcount] || $fmcount == 0 || [string is space $ftime] || $ftime == 0} {
            return 0
        }
        if {[info exists megahal(flood,$chan,$nick)]} {
            
            set utime [unixtime]
            lappend megahal(flood,$chan,$nick) $utime
            
            set fcount [llength $megahal(flood,$chan,$nick)]
            foreach line [lrange $megahal(flood,$chan,$nick) 0 end-1] {
                incr fcount -1
                if {[expr $utime - $line] < $ftime} {
                    if {$fcount >= $fmcount} {
                        return 1
                    } else {
                        return 0
                    }
                }
            }
            
        } else {
            set megahal(flood,$chan,$nick) [list [unixtime]]
            after [expr $ftime * 1000] [list ::megahal::detectflood:timer $nick $chan]
            return 0
        }
        
        return 0
        
    }
    
    proc detectflood:timer {nick chan} {
        variable megahal
        
        if {[info exists megahal(flood,$chan,$nick)]} {
            
            set lflood [split $megahal(flood) :]
            set fmcount [lindex $lflood 0]
            set ftime [lindex $lflood 1]
            
            if {[string is space $fmcount] || $fmcount == 0 || [string is space $ftime] || $ftime == 0} {
                unset megahal(flood,$chan,$nick)
                return
            }
            
            if {[llength $megahal(flood,$chan,$nick)] > 1} {
                
                set utime [unixtime]
                set ind 0
                foreach line $megahal(flood,$chan,$nick) {
                    if {[expr $utime - $line] < $ftime} {
                        break
                    }
                    incr ind
                }
                
                set megahal(flood,$chan,$nick) [lrange $megahal(flood,$chan,$nick) $ind end]
                after [expr $ftime * 1000] [list ::megahal::detectflood:timer $nick $chan]
                
            } else {
                unset megahal(flood,$chan,$nick)
            }
            
        }
        
    }
    
    proc userisignore {nick} {
        variable megahal
        
        if {[info exists megahal(limit_i,$nick)]} {
            return 1
        }
        return 0
    }
    
    proc addignore {nick} {
        variable megahal
        
        if {[info exists megahal(limit_i,$nick)]} {
            return
        } else {
            set time $megahal(time_ignore)
            set megahal(limit_i,$nick) [list [after [expr $time*1000] [list ::megahal::addignore:timer $nick]] $time]
            set megahal(limit_t,$nick) [expr [unixtime] + $time]
        }
        
    }
    
    proc addignore:timer {nick} {
        variable megahal
        
        out_debug "Снятие игнора с $nick" 2
        if {[info exists megahal(limit_i,$nick)]} {
            unset megahal(limit_i,$nick)
        }
        if {[info exists megahal(limit_t,$nick)]} {
            unset megahal(limit_t,$nick)
        }
        
    }
    
    #############################################################################################################
    # Процедуры сохранения баз
    
    proc save_dictionary {} {
        variable megahal
        
        brainsave
        set megahal(id_timer_save) [after [expr 60 * 1000 * $megahal(time_save)] [list ::megahal::save_dictionary]]
        
    }
    
    #############################################################################################################
    # Процедуры контроля за матом
    
    proc DetectMat {text} {
        variable megahal
        
        set text [string map [list "\x5d\x5b" "x" "\x7d\x7b" "x" "\x3e\x3c" "x" "\x29\x28" "x" "\x7c\x7c" "n"] $text]
        
        set text1 $text
        set text1 [string map [list A а a а B в C с c с E е e е H н K к k к M м O о o о P р p р T т U и u и X х x х Y у y у 3 з 4 ч 6 б 0 о b ь g д n п] $text1]
        set text1 [string tolower $text1]
        set text1 [remove_symbols $text1]
        set text1 [remove_duplicates $text1]
        
        set text2 $text
        if {![text_has_rus $text2]} {
            set text2 [lat2rus $text2]
            set text2 [remove_symbols $text2]
        }
        set text2 [string map [list A а a а B в C с c с E е e е H н K к k к M м O о o о P р p р T т U и u и X х x х Y у y у 3 з 4 ч 6 б 0 о b ь g д n п] $text2]
        set text2 [string tolower $text2]
        set text2 [remove_duplicates $text2]
        
        set bwecount 0
        foreach i $megahal(BadWords) {
            set i [string trim $i]
            set i [string map [list "!SP!" " " "!PD!" \. "!CO!" \: "!SQ!" \' "!QL!" \; "!QT!" \" "!LT!" \< "!GT!" \> "!LP!" \( "!RP!" \) "!LB!" \{ "!RB!" \} "!LF!" \[ "!RF!" \] "!SL!" \\] $i]
            
            set mask [split $i {}]
            if {[string is space $mask]} continue
            regsub -all { } $mask {\s*} mask
            set mask ".*\\s+$mask\\s+.*"
            if {[regexp $mask $text1] || [string match *$i* $text1] || [regexp $mask $text2] || [string match *$i* $text2]} {
                incr bwecount
            }
        }
        
        set bweecount 0
        foreach i $megahal(BadExepts) {
            set i [string trim $i]
            set i [string map [list "!SP!" " " "!PD!" \. "!CO!" \: "!SQ!" \' "!QL!" \; "!QT!" \" "!LT!" \< "!GT!" \> "!LP!" \( "!RP!" \) "!LB!" \{ "!RB!" \} "!LF!" \[ "!RF!" \] "!SL!" \\] $i]
            
            set mask [split $i {}]
            if {[string is space $mask]} continue
            regsub -all { } $mask {\s*} mask
            set mask ".*\\s+$mask\\s+.*"
            if {[regexp $mask $text1] || [string match *$i* $text1] || [regexp $mask $text2] || [string match *$i* $text2]}  {
                incr bweecount
            }
        }
        if {$bweecount < $bwecount} {return 1} else {return 0}
        
    }
    
    #############################################################################################################
    # Процедуры поднятия и убивания биндов
    
    proc binds_up {} {
        
        set curr 0
        incr curr 3
        bind pubm - * [namespace origin pubm]
        bind msgm - * [namespace origin msgm]
        
        bind EVNT -|- prerehash [namespace origin prerehash]
        
        putcmdlog "[namespace current]:: $curr binds is up"
        
    }
    
    proc prerehash { type } {
        binds_down
    }
    
    proc binds_down {} {
        
        set curr 0
        foreach line [binds] {
            if {[string range [lindex $line 4] 0 [expr [string length "[namespace current]::"] - 1]] == "[namespace current]::"} {
                incr curr
                unbind [lindex $line 0] [lindex $line 1] [lindex $line 2] [lindex $line 4]
            }
        }
        
        putcmdlog "[namespace current]:: $curr binds is down"
    }
    
    #############################################################################################################
    # Начальная загрузка
    
    load_base
    binds_up
    if {![info exists megahal(id_timer_save)]} {
        set megahal(id_timer_save) [after [expr 60 * 1000 * $megahal(time_save)] [list ::megahal::save_dictionary]]
    }
    
    if {[catch {
        loadmodule megahal
        out_debug "Модуль megahal загружен" 1
    } errMsg]} {
        out_error "Модуль не загружен ($errMsg)."
    }
    
    putlog "[namespace current]:: v$version \[$date\] by $author loaded"
    
}


